@@ -175,7 +175,8 @@ check_priors <- function(lavpartable) {
 #' @details Explain this
 stanmarg_data <- function(YX = NULL, S = NULL, YXo = NULL, N, Ng, grpnum, # data
                           miss, Np, Nobs, Obsvar, # missing
-                          ord, Nord, ordidx, contidx, nlevs, neach, # ordinal
+                          ord, Nord, ordidx, contidx, nlevs,
+                          Noent, Nordobs, OrdObsvar, # ordinal
                           Xvar, Xdatvar, Nx, # fixed.x
                           startrow, endrow, save_lvs = FALSE, do_test = TRUE,
                           Lambda_y_skeleton, # skeleton matrices
@@ -208,10 +209,12 @@ stanmarg_data <- function(YX = NULL, S = NULL, YXo = NULL, N, Ng, grpnum, # data
   dat$Obsvar <- Obsvar
   dat$ord <- ord
   dat$Nord <- Nord
+  dat$Noent <- Noent
+  dat$Nordobs <- Nordobs
+  dat$OrdObsvar <- OrdObsvar
   dat$ordidx <- ordidx
   dat$contidx <- contidx
   dat$nlevs <- nlevs
-  dat$neach <- neach
   dat$startrow <- startrow
   dat$endrow <- endrow
   dat$Xvar <- Xvar
@@ -223,13 +226,18 @@ stanmarg_data <- function(YX = NULL, S = NULL, YXo = NULL, N, Ng, grpnum, # data
   dat$YXo <- YXo
   stopifnot(nrow(dat$YX) == dat$Ntot)
 
-  dat$has_data <- dat$has_cov <- 0L
+  dat$use_suff <- 1L
+  if (ord) dat$use_suff <- 0L
+
+  dat$has_data <- 0L
   if (pri_only) {
+    dat$use_suff <- 0L
     tmparr <- array(dim = c(dat$Ng, ncol(YX) + 1, ncol(YX) + 1))
     for (i in 1:Ng) {
       tmparr[i,,] <- diag(nrow=ncol(YX) + 1)
     }
     dat$S <- tmparr
+    dat$YXbar <- array(0, dim = c(dat$Ng, ncol(YX)))
   } else {
     if (missing(YX)) {
       dat$has_data <- 0L
@@ -237,24 +245,36 @@ stanmarg_data <- function(YX = NULL, S = NULL, YXo = NULL, N, Ng, grpnum, # data
       if (!is.list(S)) stop("S must be a list")
       if (is.null(N)) stop("N must be specified if YX is missing")
       if (miss) stop("blavaan ERROR: missingness requires raw data.")
-      dat$S <- array(NA, dim=c(Ng, nrow(S), nrow(S)))
       for (i in 1:Ng) {
         dat$S[i,,] <- (N[i] - 1) * S[[i]]
       }
       dat$YX <- array(NA_real_, dim = c(dat$Ntot, ncol(S)))
       dat$YXo <- array(NA_real_, dim = c(0, ncol(YXo)))
+      dat$YXbar <- array(NA_real_, dim = c(dat$Ng, ncol(S)))
     } else {
-      dat$has_data <- 1L
-
       if (NROW(YX) != dat$Ntot) stop("blavaan ERROR: nrow(YX) != Ntot.")
-    
-      dat$S <- array(NA, dim=c(Ng, NCOL(YX) + 1, NCOL(YX) + 1))
-      for (i in 1:Ng) {
-        dat$S[i,,] <- diag(1, NCOL(YX) + 1)
-        ## not added because, if not pd, stan fails: (dat$N[i] - 1) * cov(YX[(startrow[i] : endrow[i]), , drop = FALSE]) # NB!! this multiplication is needed to use wishart_lpdf
+
+      dat$YXbar <- array(0, dim=c(Np, NCOL(YX)))
+      dat$S <- array(1, dim=c(Np, NCOL(YX) + 1, NCOL(YX) + 1))
+      dat$has_data <- 1L
+      if (length(contidx) > 0) {
+        for (i in 1:dat$Np) {
+          tmpyxbar <- colMeans(YX[(startrow[i] : endrow[i]), , drop = FALSE])
+          tmpyxbar[is.na(tmpyxbar)] <- 0
+          dat$YXbar[i,] <- tmpyxbar
+
+          tmpN <- endrow[i] - startrow[i] + 1
+          if(tmpN > 1) {
+            tmpS <- cov(YX[(startrow[i] : endrow[i]), , drop = FALSE]) * (tmpN - 1) / tmpN
+          } else {
+            tmpS <- matrix(0, NCOL(YX), NCOL(YX))
+          }
+          dat$S[i,1:NCOL(YX),1:NCOL(YX)] <- tmpS
+        }
       }
     }
   }
+
   dat$save_lvs <- save_lvs
   dat$do_test <- do_test
   
